/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

@import url(chrome://venkman/content/venkman-output-base.css);

body {
    background: black;
    color: lightgrey;
}

a.venkman-link {
    color: #7083ff;
}

a.venkman-link:visited {
    color: #4b57aa;
}

.msg-data[msg-type="ERROR"] {
    background: #550000;
    color: white;
}

.msg-data:before {
    color: yellow;
}

.msg-data[msg-type="ERROR"]:before {
    color: orange;
}

.msg-data[msg-type="ERROR"] a.venkman-link {
    color: white;
    font-weight: bold;
}

.msg-data[msg-type="STOP"]:before {
    color: pink;
}

.msg-data[msg-type="STOP"] {
    color: red;
}

.msg-data[msg-type="CONT"]:before {
    color: lightgreen;
}

.msg-data[msg-type="CONT"] {
    color: green;
}

.msg-data[msg-type="ETRACE"]:before {
    color: brown;
}
   
.msg-data[msg-type="FEVAL-IN"]:before,
.msg-data[msg-type="FEVAL-OUT"]:before {
    color: orange;
}
